@@ -15,13 +15,37 @@ class AMQ_BrokerConfig:
     uid = 'von'
     password = 'von1970'
 
+class RabbitMq_Subscriber():
+    def __init__(self, queue_name: str) -> None:
+        self.queue_name = queue_name
+        self.channel = None
+        self.prefeched_message = None
+        self.delivery_tag = 0
+
+    def CopyToFetchedMessage(self, ch, method, properties, body):
+        print(method ,'\n',  method.routing_key,'\n', body)
+
+        # self.channel.basic_ack(delivery_tag=method.delivery_tag)
+        self.channel = ch
+        self.prefeched_message = body
+        self.delivery_tag = method.delivery_tag
+
+    def FetchMessage(self) -> str:
+        result = self.prefeched_message
+        if self.prefeched_message is not None:
+            self.channel.basic_ack(delivery_tag=self.delivery_tag)
+            self.prefeched_message = None
+        return result
 
 class RabbitMqAgent():
     '''
     To learn:  What is channel indeed ?
     '''
     def __init__(self) -> None:
-        self.declaed_queues=[]
+        # self.declaed_queues=[]
+        self.subscribers=[RabbitMq_Subscriber('Nothing')]
+        self.subscribers.clear
+        self.prefeched_message = None
 
     def connect_to_broker(self, broker_config: AMQ_BrokerConfig) -> None:
         self.serverConfig = broker_config
@@ -33,12 +57,12 @@ class RabbitMqAgent():
     def SpinOnce(self):
         self.blocking_connection.process_data_events()
     
-    def FetchMessage(self) -> str:
-        result = self.prefeched_message
-        if self.prefeched_message is not None:
-            self.channel.basic_ack(delivery_tag=self.delivery_tag)
-            self.prefeched_message = None
-        return result
+    # def FetchMessage(self) -> str:
+    #     result = self.prefeched_message
+    #     if self.prefeched_message is not None:
+    #         self.channel.basic_ack(delivery_tag=self.delivery_tag)
+    #         self.prefeched_message = None
+    #     return result
 
     def reconnect_to_broker(self):
         broker_config = self.serverConfig
@@ -67,9 +91,9 @@ class RabbitMqAgent():
 
     def PublishBatch(self, queue_name:str, payloads:list):
         print("[Info] RabbitMqAgent.PublishBatch(), queue_name=, payload= ", queue_name, payloads)
-        if not (queue_name in self.declaed_queues):
-            self.channel.queue_declare(queue=queue_name)
-            self.declaed_queues.append(queue_name)
+        # if not (queue_name in self.declaed_queues):
+        #     self.channel.queue_declare(queue=queue_name)
+        #     self.declaed_queues.append(queue_name)
         for pp in payloads:
             self.channel.basic_publish(exchange = '',
                             routing_key = queue_name,
@@ -93,11 +117,24 @@ class RabbitMqAgent():
                         routing_key = queue_name,
                         body = img_pub)
 
-    def CopyToFetchedMessage(self, ch, method, properties, body):
-        print('RabbitMqAgent::callback_example()  mq Received ' ,  method.routing_key, body)
-        # self.channel.basic_ack(delivery_tag=method.delivery_tag)
-        self.prefeched_message = body
-        self.delivery_tag = method.delivery_tag
+    def __FindSubscriber(self, queue_name:str) -> RabbitMq_Subscriber:
+        for subscriber in self.subscribers:
+            if subscriber.queue_name == queue_name:
+                return subscriber
+        return None
+
+    def fetch_message(self, queue_name:str) -> str:
+        subscriber = self.__FindSubscriber(queue_name)
+        if subscriber is None:
+            return None
+        else:
+            return subscriber.FetchMessage()
+
+    # def CopyToFetchedMessage(self, ch, method, properties, body):
+    #     # print('RabbitMqAgent::callback_example()  mq Received ' ,  method.routing_key, body)
+    #     # self.channel.basic_ack(delivery_tag=method.delivery_tag)
+    #     self.prefeched_message = body
+    #     self.delivery_tag = method.delivery_tag
 
 
     def Subscribe(self, queue_name:str, callback=None):
@@ -105,14 +142,18 @@ class RabbitMqAgent():
         call back examole def callback_main(self, ch, method, properties, body):
         If using FetchMessage(), ignore the callback
         '''
-        if not (queue_name in self.declaed_queues):
-            self.channel.queue_declare(queue=queue_name, durable=True)
-            self.declaed_queues.append(queue_name)
-
+        # if not (queue_name in self.declaed_queues):
+            # self.channel.queue_declare(queue=queue_name, durable=True)
+            # self.declaed_queues.append(queue_name)
+        new_subscriber = RabbitMq_Subscriber(queue_name)
+        new_subscriber.channel = self.channel
+        self.subscribers.append(new_subscriber)
         var_callback = callback
         if callback is None:
-            var_callback = self.CopyToFetchedMessage
-        self.channel.basic_consume(queue=queue_name, on_message_callback=var_callback, auto_ack=False)
+            # var_callback = self.CopyToFetchedMessage
+            var_callback = new_subscriber.CopyToFetchedMessage
+        # self.channel.basic_consume(queue=queue_name, on_message_callback=var_callback, auto_ack=False)
+        new_subscriber.channel.basic_consume(queue=queue_name, on_message_callback=var_callback, auto_ack=False)
 
     def RabbitMQ_publish_tester(self):
         i = 0
@@ -137,12 +178,20 @@ if __name__ == '__main__':
     # img = cv2.imread("nocommand.jpg")
     # g_amq.publish_cv_image("test" , img)
 
-    g_amq.Subscribe(queue_name='twh_221109_request')
+    g_amq.Subscribe(queue_name='twh_221109_deposit')
+    g_amq.Subscribe(queue_name='twh_221109_withdraw')
     while True:
         g_amq.SpinOnce()
-        xx = g_amq.FetchMessage()
+        xx = g_amq.fetch_message('twh_221109_deposit')
+        # xx = ss.FetchMessage()
+        if xx is not None:
+            print(xx)
+            # time.sleep(1)
+
+        xx = g_amq.fetch_message('twh_221109_withdraw')
         if xx is not None:
             print(xx)
             time.sleep(1)
 
+
     
\ No newline at end of file
